/***

mMonkeyEntry - DUIS-Main.m
by Leechbite

main script for the DUI interface.

***/

#include <lib/std.mi>
#include "attribs.m"
#include "utilities.m"
#include "dui-utilities.m"

function setNewGroup(String groupid);
function openDUIWindow(string duiWinID, string windowName, int min_w, int min_h, int max_w, int max_h);
function closeDUIWindow(string duiWinID);
function closeDUIWindowContent(string duiWinID, string windowName);
function autoResizeDUIWindow(group duiWindowMain, guiObject duiWindowHole, int min_w, int min_h, int max_w, int max_h);
function ConfigAttribute getDUIwinAttrib(string duiWinID);
function string getMainDUIWin(string duiWinID);
function initDUIAttrib(string windowName);

function refreshDUILayout(); // refreshes the DUI dimensions
function int getMainWidth(); // computes current width of main DUI window based on sideWidth
function int getMinWidth(); // computes the proper minimum layout width for the current displayed windows
function int getMinHeight(); // ditto.. except its for height.

Global group frameGroup;
Global layout duiLayout;
Global container duiContainer;

Global group duiWindowMain,duiWindowMain2,duiWindowSide,duiWindowSide2;
Global guiObject duiWindowHoleMain,duiWindowHoleMain2,duiWindowHoleSide,duiWindowHoleSide2;
Global string duiLay_min_w, duiLay_min_h, duiLay_max_w, duiLay_max_h;
Global int main_min_w, main_min_h, main_max_w, main_max_h;
Global int main2_min_w, main2_min_h, main2_max_w, main2_max_h;
Global int side_min_w, side_min_h, side_max_w, side_max_h;
Global int side2_min_w, side2_min_h, side2_max_w, side2_max_h;
Global int sideWidth;
Global boolean localDUIOpenClose = FALSE, resizerDown = FALSE;

Global timer sendDUIDelay;
Global string sendDUIWindowName, sendDUICommand, sendDUIParam;

Global string windowList;

global group resizerGroup;
global guiObject resizerHandle;

global boolean localFocus = FALSE;
global boolean lastFocus = FALSE;

Global ConfigAttribute duiAttrib;

System.onScriptLoaded() {
  initAttribs();
	
  frameGroup = getScriptGroup();
  duiLayout = frameGroup.getParentLayout();
  duiContainer = duiLayout.getContainer();
  
  duiWindowMain = frameGroup.findObject("dui.main");
  duiWindowSide = frameGroup.findObject("dui.side");
  
  duiWindowMain.hide();
  duiWindowSide.hide();

  duiWindowHoleMain = duiWindowMain.findObject("dui.main.content");
  duiWindowHoleSide = duiWindowSide.findObject("dui.side.content");
  
  resizerGroup = frameGroup.findObject("dui.resizer");
  resizerHandle = resizerGroup.findObject("resizer.handle");
  sideWidth = getPrivateInt(getSkinName(),"Side DUI Window Width",150);
  
  duiLay_min_w = duiLayout.getXMLParam("minimum_w");
  duiLay_min_h = duiLayout.getXMLParam("minimum_h");
  duiLay_max_w = duiLayout.getXMLParam("maximum_w");
  duiLay_max_h = duiLayout.getXMLParam("maximum_h");
  
 
  main_min_w = 50; main_min_h = 50; main_max_w = 2000; main_max_h = 2000;
  main2_min_w = 50; main2_min_h = 50; main2_max_w = 2000; main2_max_h = 2000;
  side_min_w = 50; side_min_h = 50; side_max_w = 2000; side_max_h = 2000;
  side2_min_w = 50; side2_min_h = 50; side2_max_w = 2000; side2_max_h = 2000;
    
  sendDUIDelay = new Timer;
  sendDUIDelay.setDelay(1);

  windowList = getPrivateString(getSkinName(), "DUI Window List", "");
  int c = 0;
  string winName = "";
  do {
    winName = getToken(windowList, ",", c);
    if (winName!="") {
      initDUIAttrib(winName);
    }
    c++;
  } while (winName!="");
  
  //refreshDUILayout();
}

onReceiveDUICommand(string winName, string command, string param) {
  windowName = strlower(winName);
  
  //setTempText(command+"|"+winName);
  
  string param1 = strlower(getToken(param,",",0));
  string param2 = strlower(getToken(param,",",1));
  string param3 = strlower(getToken(param,",",2));
  string param4 = strlower(getToken(param,",",3));
  string param5 = strlower(getToken(param,",",4));
  
  if (windowName!=MAINID) {
    if (command == OPEN_DUI_REQUEST ) {

      closeDUIWindowContent(param1, windowName);
      openDUIWindow(param1, windowName, stringToInteger(param2), stringToInteger(param3), stringToInteger(param4), stringToInteger(param5));
      sendDUICommand(MAINID, OPEN_DUI_GRANTED, windowName+","+getMainDUIWin(param1));
        
    } else if (command == DETACH_WINDOW_REQUEST) {
    
      closeDUIWindow(param1);
      sendDUICommand(MAINID, DETACH_WINDOW_GRANTED, windowName);

    } else if (command == CLOSE_DUI_WINDOW) {

      ConfigAttribute duiwinAttrib = getDUIwinAttrib(param1);
      if (duiwinAttrib) {
	    string attribdata = strlower(duiwinAttrib.getData());
		string winName = strlower(windowName);

        if ((attribdata == winName) || (winName == "all")) {
          closeDUIWindow(param1);
        }
      } else closeDUIWindow(param1);
      
    } else if (command == ADD_TO_DUI_LIST) {
      if ((strsearch(windowList,winName) < 0) || (windowList == "")) {

	    if (windowList == "") {
	      windowList = winName;
    	} else {
	      windowList = windowList + "," + winName;
        }
	    setPrivateString(getSkinName(), "DUI Window List", windowList);
	    initDUIAttrib(winName);
      }
    } else if (command == SET_FOCUS) {
	  if (param1 == "all") param1 = strlower(MAINID);
  	  if (param1 == strlower(MAINID)) {
	    localFocus = TRUE;
	    duiLayout.setFocus();
      }
    }
  }
}

initDUIAttrib(string windowName) {
	duiAttrib = custom_page_windows.newAttribute("Attach "+windowName+" window", "1");
}

ConfigAttribute getDUIwinAttrib(string duiWinID) {
  if (duiWinID == DUI_WIN_MAIN) {
    return dui_win_main_content_attrib;
  } else if (duiWinID == DUI_WIN_SIDE) {
    return dui_win_side_content_attrib;
  }
}

string getMainDUIWin(string duiWinID) {
  if (duiWinID == DUI_WIN_MAIN) {
    return MAINID+","+duiLayout.getID()+","+duiWindowMain.getID()+","+duiWindowHoleMain.getID();
  } else if (duiWinID == DUI_WIN_SIDE) {
    return MAINID+","+duiLayout.getID()+","+duiWindowSide.getID()+","+duiWindowHoleSide.getID();
  }
}

openDUIWindow(string duiWinID, string windowName, int min_w, int min_h, int max_w, int max_h) {
  if (duiWinID == DUI_WIN_MAIN) {
  
    if (min_w <= 0) main_min_w = 50; else main_min_w = min_w;
    if (min_h <= 0) main_min_h = 50; else main_min_h = min_h;
    if (max_w < min_w) main_max_w = 2000; else main_max_w = max_w;
    if (max_h < min_h) main_max_h = 2000; else main_max_h = max_h;
    autoResizeDUIWindow(duiWindowMain, duiWindowHoleMain, main_min_w, main_min_h, main_max_w, main_max_h);
    
    localDUIOpenClose = TRUE;
    
    duiWindowHoleMain.show();
    duiWindowMain.show();
    
    localDUIOpenClose = FALSE;
    
  } else if (duiWinID == DUI_WIN_SIDE) {
    if (min_w <= 0) side_min_w = 50; else side_min_w = min_w;
    if (min_h <= 0) side_min_h = 50; else side_min_h = min_h;
    if (max_w < min_w) side_max_w = 2000; else side_max_w = max_w;
    if (max_h < min_h) side_max_h = 2000; else side_max_h = max_h;
    
    localDUIOpenClose = TRUE;
	  
    duiWindowHoleSide.show();
    duiWindowSide.show();
    
    localDUIOpenClose = FALSE;
  }
  
  ConfigAttribute duiwinAttrib = getDUIwinAttrib(duiWinID);
  if (duiwinAttrib) duiwinAttrib.setData(windowName);
  
  refreshDUILayout();

}

closeDUIWindow(string duiWinID) {
  localDUIOpenClose = TRUE;
  if (duiWinID == DUI_WIN_MAIN) {
    duiWindowHoleMain.hide();
    duiWindowMain.hide();
  } else if (duiWinID == DUI_WIN_SIDE) {
    duiWindowHoleSide.hide();
    duiWindowSide.hide();
  }
  localDUIOpenClose = FALSE;
  ConfigAttribute duiwinAttrib = getDUIwinAttrib(duiWinID);
  if (duiwinAttrib) duiwinAttrib.setData("");
  
  refreshDUILayout();
}

closeDUIWindowContent(string duiWinID, string windowName) {
  ConfigAttribute duiwinAttrib = getDUIwinAttrib(duiWinID);
  
  if (duiwinAttrib) {

    if (duiwinAttrib.getData()=="") return;
	  
	string attribdata = strlower(duiwinAttrib.getData());
	string winName = strlower(windowName);
	
    if (attribdata != winName) {

       sendDUIWindowName = MAINID;
       sendDUICommand = CLOSE_DUI_WINDOW;
       sendDUIParam = duiwinAttrib.getData();
       sendDUIDelay.start();

    }
  }
}


// has to coded still :D
duiWindowMain.onSetVisible(boolean on) {
  refreshDUILayout();
  if (localDUIOpenClose) return;
  if (on) {
	return;
  } else {
	return;
  }
}

duiWindowSide.onSetVisible(boolean on) {
  refreshDUILayout();
  if (localDUIOpenClose) return;
  if (on) {
	return;
  } else {
	return;
  }
}

sendDUIDelay.onTimer() {
  stop();
  sendDUICommand(sendDUIWindowName, sendDUICommand, sendDUIParam);
}

autoResizeDUIWindow(group duiWindow, guiObject duiWindowHole, int min_w, int min_h, int max_w, int max_h) {
  int mainWidth = duiWindow.getWidth();
  int mainHeight = duiWindow.getHeight();
  int newWidth, newHeight;

  if (mainWidth > max_w) 
    duiWindowHole.setXMLParam("w", integerToString(max_w));
  else if (mainWidth < min_w)
    duiWindowHole.setXMLParam("w", integerToString(mainWidth));
  else
    duiWindowHole.setXMLParam("w", integerToString(mainWidth));
    
  if (mainHeight > max_h) 
    duiWindowHole.setXMLParam("h", integerToString(max_h));
  else if (mainHeight < min_h)
    duiWindowHole.setXMLParam("h", integerToString(mainHeight));
  else
    duiWindowHole.setXMLParam("h", integerToString(mainHeight));
    
  refreshDUILayout();
}

duiWindowMain.onResize(int wx, int wy, int ww, int wh) {
  autoResizeDUIWindow(duiWindowMain, duiWindowHoleMain, main_min_w, main_min_h, main_max_w, main_max_h);
}

duiWindowSide.onResize(int wx, int wy, int ww, int wh) {
  autoResizeDUIWindow(duiWindowSide, duiWindowHoleSide, side_min_w, side_min_h, side_max_w, side_max_h);
}

// attrib toggles every 0.5 sec
genTimerAttrib.onDataChanged() {

  if (localFocus) { localFocus = FALSE; return; }

  if ((lastFocus == FALSE) && (duiLayout.isActive())) {
    sendDUICommand(MAINID, SET_FOCUS, "all");
    duiLayout.setFocus();	

  }
  lastFocus = duiLayout.isActive();
  
}

refreshDUILayout() {
  int main_guix = stringToInteger(duiWindowMain.getXMLParam("x"));
  int main_guiy = stringToInteger(duiWindowMain.getXMLParam("y"));
  int main_guiw = stringToInteger(duiWindowMain.getXMLParam("w"));
  int main_guih = stringToInteger(duiWindowMain.getXMLParam("h"));
  
  // resets duiLayout to proper dimensions
  int duiWidth = duiLayout.getWidth();
  int duiHeight = duiLayout.getHeight();
  int minWidth = getMinWidth();
  int minHeight = getMinHeight();

  if (duiWidth < minWidth) duiWidth = minWidth;
  if (duiHeight < minHeight) duiHeight = minHeight;
  
    duiLayout.setXMLParam("minimum_w",integerToString(minWidth));
    duiLayout.setXMLParam("minimum_h",integerToString(minHeight));
    if (!duiWindowMain.isVisible() && !duiWindowSide.isVisible()) {
      duiLayout.setXMLParam("maximum_w",duiLay_max_w);
      duiLayout.setXMLParam("maximum_h",integerToString(minHeight));
      duiLayout.resize(duiLayout.getLeft(),duiLayout.getTop(), duiWidth, minHeight);
    } else {
      duiLayout.setXMLParam("maximum_w",duiLay_max_w);
      duiLayout.setXMLParam("maximum_h",duiLay_max_h);
      duiLayout.resize(duiLayout.getLeft(),duiLayout.getTop(), duiWidth, duiHeight);
    }
    
  if (sideWidth < side_min_w) sideWidth = side_min_w;
  
  int mainw = getMainWidth();
  if (mainw < main_min_w) sideWidth = sideWidth + mainw - main_min_w;
  
  if (!duiWindowMain.isVisible()) {
	duiWindowSide.setXMLParam("x","0");
	duiWindowSide.setXMLParam("w","0");
	duiWindowSide.setXMLParam("relatx","0");
	duiWindowSide.setXMLParam("relatw","1");
	resizerGroup.hide();
  } else if (!duiWindowSide.isVisible()) {
	duiWindowMain.setXMLParam("w","0");
	resizerGroup.hide();
  } else {
	duiWindowMain.setXMLParam("w",integerToString(-sideWidth-resizerGroup.getWidth()));
	duiWindowSide.setXMLParam("x",integerToString(-sideWidth));
	duiWindowSide.setXMLParam("w",integerToString(sideWidth));
	duiWindowSide.setXMLParam("relatx","1");
	duiWindowSide.setXMLParam("relatw","0");
	resizerGroup.show();
	resizerGroup.setXMLParam("x",integerToString(-sideWidth-resizerGroup.getWidth()));
  }
  

}

resizerHandle.onLeftButtonDown(int x, int y) {
  resizerDown = TRUE;
  setXMLParam("resize","");
  duiLayout.setXMLParam("minimum_w",integerToString(duiLayout.getWidth()));
  duiLayout.setXMLParam("minimum_h",integerToString(duiLayout.getHeight()));
  duiLayout.setXMLParam("maximum_w",integerToString(duiLayout.getWidth()));
  duiLayout.setXMLParam("maximum_h",integerToString(duiLayout.getHeight()));
}

resizerHandle.onMouseMove(int x, int y) {
  if (!resizerDown) return;
  sideWidth = (frameGroup.getLeft()+frameGroup.getWidth())-x;
  refreshDUILayout();
}


resizerHandle.onLeftButtonUp(int x, int y) {
  resizerDown = FALSE;
  setXMLParam("resize","right");
  refreshDUILayout();
}

int getMainWidth() {
  return frameGroup.getWidth()-sideWidth-resizerGroup.getWidth();
}

int getMinWidth() {
  int mainmin = main_min_w;
  if (mainmin < main2_min_w) mainmin = main2_min_w;
  if (!duiWindowMain.isVisible()) mainmin = 0;
  
  int sidemin = side_min_w;
  if (sidemin < side2_min_w) sidemin = side2_min_w;
  if (!duiWindowSide.isVisible()) sidemin = 0;
	
  int minw = (duiLayout.getWidth()-frameGroup.getWidth()) + mainmin + sidemin;
  if (minw < stringToInteger(duiLay_min_w)) minw = stringToInteger(duiLay_min_w);
  
  return minw;
}

int getMinHeight() {
  int minh = main_min_h;
  if (!duiWindowMain.isVisible()) minh = 0;
  
  if ((duiWindowSide.isVisible()) && (minh < side_min_h)) minh = side_min_h;

  minh = (duiLayout.getHeight()-frameGroup.getHeight()) + minh;

  return minh;
}